use std::{collections::VecDeque, sync::Arc};

use super::{
    command_base::{CommandDeserializer, DeserializedCommand},
    mix_effects::program_input::ProgramInputDeserializer,
    tally_by_source::TallyBySourceDeserializer,
};

pub fn deserialize_commands(payload: &[u8]) -> VecDeque<Arc<dyn DeserializedCommand>> {
    let mut parsed_commands = VecDeque::new();
    let mut head = 0;

    while payload.len() > head + 8 {
        let length = u16::from_be_bytes([payload[head], payload[head + 1]]) as usize;
        let Ok(name) = String::from_utf8(payload[(head + 4)..(head + 8)].to_vec()) else {
            break;
        };

        if length < 8 {
            break;
        }

        log::debug!("Received command {} with length {}", name, length);

        if let Some(deserializer) = command_deserializer_from_string(name.as_str()) {
            let deserialized_command = deserializer.deserialize(&payload[head + 8..head + length]);
            log::debug!("Received {:?}", deserialized_command);
            parsed_commands.push_back(deserialized_command);
        }

        head += length;
    }

    parsed_commands
}

fn command_deserializer_from_string(command_str: &str) -> Option<Box<dyn CommandDeserializer>> {
    match command_str {
        "PrgI" => Some(Box::<ProgramInputDeserializer>::default()),
        "TlSr" => Some(Box::<TallyBySourceDeserializer>::default()),
        _ => None,
    }
}
