use std::{collections::HashMap, sync::Arc};

use super::command_base::{CommandDeserializer, DeserializedCommand};

#[derive(Debug)]
pub struct TallySource {
    pub program: bool,
    pub preview: bool,
}

#[derive(Debug, new)]
pub struct TallyBySource {
    pub sources: HashMap<u16, TallySource>,
}

impl DeserializedCommand for TallyBySource {
    fn apply_to_state(&self, state: &mut crate::state::AtemState) -> Vec<String> {
        todo!()
    }
}

#[derive(Default)]
pub struct TallyBySourceDeserializer {}

impl CommandDeserializer for TallyBySourceDeserializer {
    fn deserialize(&self, buffer: &[u8]) -> Arc<dyn DeserializedCommand> {
        let source_count = u16::from_be_bytes([buffer[0], buffer[1]]) as usize;

        log::debug!("{:?}", buffer);
        log::debug!("Source count: {}", source_count);

        let mut sources = HashMap::new();
        for i in 0..source_count {
            let source_byte_offset = 2 + (i * 3);
            let source =
                u16::from_be_bytes([buffer[source_byte_offset], buffer[source_byte_offset + 1]]);
            let value_byte_offset = 4 + (i * 3);
            let value = u8::from_be_bytes([buffer[value_byte_offset]]);
            sources.insert(
                source,
                TallySource {
                    program: (value & 0x01) > 0,
                    preview: (value & 0x02) > 0,
                },
            );
        }

        Arc::new(TallyBySource { sources })
    }
}
