use std::{sync::Arc, time::Duration};

use atem_connection_rs::{
    atem_lib::atem_socket::AtemSocket,
    commands::{
        command_base::{BasicWritableCommand, SerializableCommand},
        mix_effects::program_input::ProgramInput,
    },
};

use clap::Parser;
use color_eyre::Report;
use tokio::{task::yield_now, time::sleep};

/// ATEM Rust Library Test App
#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
struct Args {
    /// IP of the ATEM to connect to
    #[arg(short, long)]
    ip: String,
}

#[tokio::main]
async fn main() {
    let args = Args::parse();

    setup_logging().unwrap();

    let switch_to_source_1 = ProgramInput::new(0, 1);
    let switch_to_source_2 = ProgramInput::new(0, 2);

    let atem = tokio::sync::RwLock::new(AtemSocket::default());
    let atem = Arc::new(atem);
    let atem_thread = atem.clone();
    tokio::spawn(async move {
        loop {
            atem_thread.write().await.tick().await;

            yield_now().await;
        }
    });
    atem.write().await.connect(args.ip, 9910).await.ok();

    let mut tracking_id = 0;
    loop {
        tracking_id += 1;
        sleep(Duration::from_millis(5000)).await;
        atem.write()
            .await
            .send_command(
                &switch_to_source_1.payload(atem_connection_rs::enums::ProtocolVersion::Unknown),
                switch_to_source_1.get_raw_name(),
                tracking_id,
            )
            .await;
        tracking_id += 1;
        sleep(Duration::from_millis(5000)).await;
        atem.write()
            .await
            .send_command(
                &switch_to_source_2.payload(atem_connection_rs::enums::ProtocolVersion::Unknown),
                switch_to_source_2.get_raw_name(),
                tracking_id,
            )
            .await;
        tracking_id += 1;
    }
}

fn setup_logging() -> Result<(), Report> {
    if std::env::var("RUST_LIB_BACKTRACE").is_err() {
        std::env::set_var("RUST_LIB_BACKTRACE", "1");
    }
    color_eyre::install()?;

    if std::env::var("RUST_LOG").is_err() {
        std::env::set_var("RUST_LOG", "debug");
    }
    env_logger::init();

    Ok(())
}
